package edu.uoc.rural;

import edu.uoc.rural.R;
import edu.uoc.rural.data.House;
import edu.uoc.rural.data.ListHouses;
import edu.uoc.rural.util.Services;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;

public class ListShow extends ListActivity{
	
	public static final int NAME_ORDER = Menu.FIRST;
	private static String TAG = "List";
	private ArrayAdapter<String> dataAdapter;

	// Called when the activity is first created.
	public void onCreate(Bundle bundle) {
		super.onCreate(bundle);
		
		
		//Creating list houses
		ListHouses listHouses = new ListHouses(this.getApplicationContext());
		
		try{
			//Calling to create and populate list if its empty
			if(listHouses.isEmpty()){
				Services s = new Services();
				listHouses.createList(s,new Handler());
				new LoadListHouses().execute(s);
			}
			else //Getting names if list is not empty
			{
				String[] names = listHouses.getNames();
				dataAdapter = new ArrayAdapter<String>(this,R.layout.list,names);
				
				//Showing list with house names
				this.setListAdapter(dataAdapter);
				
			}
		}catch (Exception e){
			Log.e(TAG,"onCreate(): error getting list information"+e.toString());
			showAlert();
		}
	}
	
	
	//Async task to load houses list from openerp database
	private class LoadListHouses extends AsyncTask<Services,Void,Boolean>{
		
		private ProgressDialog progDialog = new ProgressDialog(ListShow.this);
		ListHouses listHouses = new ListHouses(ListShow.this);
		 
		//This function is called before main background task
		protected void onPreExecute() {
			//Creating progress dialog and showing it
			progDialog.setMessage("Carregant. Espereu si us plau ... ");
			progDialog.show();
        }
		
		//Main task, populates house basic info
		protected Boolean doInBackground(Services... s) {
			 try{
				 Services serv = (Services)s[0];
				 //Checking if list is empty or had been populated before
		        if(listHouses.isEmpty()){
		            listHouses.populateBasicInfo(serv);	
		        }
		     }catch(Exception e){
		        Log.e(TAG,"run(): error running thread"+e.toString());
		        cancel(true);
		        progDialog.dismiss();
		        showAlert();
		        return false;
		     }
		     return true;
	     }

		//This task is called after main task
	     protected void onPostExecute(Boolean b) {
	    	 progDialog.dismiss();
	    	
	    	//Initializing string names vector
	 		String[] names;
	    	 
	    	 //Filling vector with names if list is not empty
	    	 if(listHouses!= null && !listHouses.isEmpty()){
	    		 names = listHouses.getNames();
	    		 
	    		//Setting adapter
	 	 		dataAdapter = new ArrayAdapter<String>(ListShow.this,R.layout.list,names);
	 	 		
	 	 		//Showing list with house names
	 	 		ListShow.this.setListAdapter(dataAdapter);
	    	 }else{
	    		 showAlert();
	    	 }
	     }

	}
	
	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		super.onListItemClick(l, v, position, id);
		
		//Get the houses list
		ListHouses listHouses = new ListHouses(this.getApplicationContext());
		
		//Calling TabHouse activity only if list is not empty
		if(!listHouses.isEmpty()){
			// Get the house that was clicked
			House house = listHouses.getList().get(position);
		
			//Creating TabHouse Intent
			Intent tabHouse = new Intent(this,TabHouse.class);
			
			//Creating bundle to pass parameter
			Bundle bund = new Bundle();
			bund.putInt("houseId", house.getId());
			tabHouse.putExtras(bund);
			
			//Calling to next activity
			startActivity(tabHouse);
		}
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {    
		MenuInflater inflater = getMenuInflater();    
		
		//Creating menu with order by distance and name options
		inflater.inflate(R.layout.listmenu, menu);    
		return true;
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {    
		//Creating instance of listHouses
		ListHouses listHouses = new ListHouses(this.getApplicationContext());
		//Initializing string names vector
 		String[] names = {"No hi ha cases disponibles"};
		
		//Handle item selection
		switch (item.getItemId()) {    
			case R.id.buttonByName:        
				
				//Ordering list houses by name
				listHouses.orderListByName();
				names = listHouses.getNames();
				this.dataAdapter = new ArrayAdapter<String>(ListShow.this,R.layout.list,names);
				this.dataAdapter.notifyDataSetChanged();
				return true;
				
			case R.id.buttonByDistance:
			
				//Ordering list houses by distance
				listHouses.orderListByDistance();
				names = listHouses.getNames();
				this.dataAdapter = new ArrayAdapter<String>(ListShow.this,R.layout.list,names);
				this.dataAdapter.notifyDataSetChanged();
	
				return true;
		}
		return true;
	} 
	
	private void showAlert(){
    	AlertDialog alert = new AlertDialog.Builder(this).create();
		alert.setTitle("Avís: error obtenint dades");
		alert.setMessage("S'ha produït un error obtenint les dades");
		alert.setButton("Tanca", new DialogInterface.OnClickListener() {
		   public void onClick(DialogInterface dialog, int which) {
	    		 finish();
		   }
		});
		alert.setIcon(R.drawable.icon);
		alert.show();
    }
	
}