package edu.uoc.rural;

import java.util.List;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.google.android.maps.MapActivity; 
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

import edu.uoc.rural.R;
import edu.uoc.rural.Maps.*;
import edu.uoc.rural.data.House;
import edu.uoc.rural.data.ListHouses;
import edu.uoc.rural.util.Services;

public class Map extends MapActivity {
	
	private static String TAG ="Map";
	
	private MapMainOverlay mainOverlay;
	private MapView mapView;
	private ListHouses listHouses;
	private List<Overlay> mapOverlays;
	private ListHouseItem houseItem;
  
	public void onCreate(Bundle savedInstanceState) {  
	    super.onCreate(savedInstanceState);  
	    
	    //Setting layout for this view
	    setContentView(R.layout.mapa);   
	    mapView = (MapView) findViewById(R.id.mapview);
	    
	    //Adding zoom controls
	    mapView.setBuiltInZoomControls(true);
	    
	    //Creating listHouses
	    listHouses = new ListHouses(this.getApplicationContext());
	    
	    int houseId = -1;
	    
	    //Recovering parameters. Bundle exists if invocation comes from house info
	    //Bundle is null if invocation comes from main screen
	    Bundle bundle = this.getIntent().getExtras();
	    if(bundle!=null){
	    	houseId = bundle.getInt("houseId", 0);
	    }
	    
	    //Getting map overlays
	    mapOverlays = mapView.getOverlays();
	    
	    if(houseId>0){ //Invocation comes from house info
			//Finding house by id in houses list
		    House house = listHouses.findById(houseId);
		    
		    //Creating drawable with single house icon
		    Drawable drawable = this.getResources().getDrawable(R.drawable.house_icon);
	      	drawable.setBounds(0, 0, drawable.getIntrinsicWidth(),drawable.getIntrinsicHeight());
	      	houseItem = new ListHouseItem(drawable,this);
	      	
	      	//Adding house overlay 
		    houseItem.addOverlay(house.getHouseOverlay());
		    
		    //Setting zoom and point
	        mapView.getController().setCenter(house.getHouseOverlay().getGeopoint());
	        mapView.getController().setZoom(11);
		    
	        //Adding main overlay to list overlays
	        mapOverlays.clear();
		    mapOverlays.add(houseItem);  
		    mapView.invalidate();
	    }else{
		    //Populating list houses with map information
	        try{
	        	Services s = new Services();
	        	//Checking if list is empty or had been populated before
	        	if(listHouses.isEmpty()) {
	        		listHouses.createList(s,new Handler());
	        		listHouses.populateBasicInfo(s);
	        	}
	        }catch(Exception e){
	        	Log.e(TAG,"onCreate():error populating map information"+e.toString());
	        	showAlert();
	        }
		    
	        Drawable drawable = this.getResources().getDrawable(R.drawable.hostel);
	      	drawable.setBounds(0, 0, drawable.getIntrinsicWidth(),drawable.getIntrinsicHeight());
	      	houseItem = new ListHouseItem(drawable,this);
	        
		    //Creating main overlay (Location management and Circle drawing
		    mainOverlay = new MapMainOverlay(this,mapOverlays,mapView,new Handler(){
		    	@Override
		    	public void handleMessage(Message m){
		    		if(m.what == MapMainOverlay.MOTION_STOP){
		    			//Adding main overlay to list overlays
		    	        mapOverlays.clear();
		    		    mapOverlays.add(mainOverlay);
		    			//Getting houses inside the circle, nearHouses
		    			try{
			    		    List<MapHouseItem> nearHouses = listHouses.getNearHousesOverlay(mainOverlay.getRadius());
			    		    
			    			if(nearHouses!=null && nearHouses.size()>0){
			    				for(int i=0;i<nearHouses.size();i++){
			    					houseItem.addOverlay(nearHouses.get(i));
			    				}
			    				mapOverlays.add(houseItem);
			    			}
			    			mapView.invalidate();
		    			}catch(Exception e){
		    				Log.e(TAG,"Error handleMessage() in Motion stop:"+e.toString());
		    			}
		    		}else if (m.what == MapMainOverlay.LOCATION_CHANGED){
		    			try{
			    			//Re-calculating house distances from user geopoint
			    			listHouses.updateDistances(mainOverlay.getPoint());
			    			mapOverlays.clear();
			    			mapOverlays.add(mainOverlay);
			    			mapView.invalidate();
		    			}catch(Exception e){
		    				Log.e(TAG,"Error handeMessage(): in location changed"+e.toString());
		    			}	
		    		}	
		    	}
		    });
		    
		    //Updating house distances
		    listHouses.updateDistances(mainOverlay.getPoint());
		
		    //Setting zoom and point
	        mapView.getController().setCenter(mainOverlay.getPoint());
	        mapView.getController().setZoom(10);
		    
	        //Adding main overlay to list overlays
	        mapOverlays.clear();
		    mapOverlays.add(mainOverlay);      
	    }
	}
  
	@Override protected boolean isRouteDisplayed() {  
	  return false;
	}
	
	private void showAlert(){
    	AlertDialog alert = new AlertDialog.Builder(this).create();
		alert.setTitle("Avís: error obtenint dades");
		alert.setMessage("S'ha produït un error obtenint les dades");
		alert.setButton("Tanca", new DialogInterface.OnClickListener() {
		   public void onClick(DialogInterface dialog, int which) {
	    		 finish();
		   }
		});
		alert.setIcon(R.drawable.icon);
		alert.show();
    }
}