package edu.uoc.rural.Maps;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Paint.Cap;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.MotionEvent;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.Projection;

public class MapMainOverlay extends Overlay{
	private static String TAG ="MapMainOverlay";
	public static final int MOTION_STOP = 50;
	public static final int LOCATION_CHANGED = 51;
	private Context context;
	private GeoPoint point;

	//Radius attributes
	private float radiusInPixels;
	private int radiusInMeters = 5000;

	//Center coordinates
	private float centerXInPixels;
	private float centerYInPixels;

	private int status = 0;

	//Small circle attributes
	private float smallCircleX;
	private float smallCircleY;
	private float smallCircleRadius = 10;

	private float angle = 0;

	private Handler handler;

	private List<LocationListener> listeners;

	public MapMainOverlay(Context context,List<Overlay> mapOverlays,MapView mapVieW, Handler handler) {
		Log.d(TAG, "MapMainOverlay(): creatting");
		this.context = context;
		this.handler = handler;
		
		//Creating location manager
		LocationManager locationManager = (LocationManager) this.context
				.getSystemService(Context.LOCATION_SERVICE);
		List<String> providers = locationManager.getProviders(true);
		
		//Creating listeners
		listeners = new LinkedList<LocationListener>();
		for (int i = 0; i < providers.size(); i++) {
			LocationListener ll = new LocationListener() {
				@Override
				public void onLocationChanged(Location location) {
					update(location);
				}

				@Override
				public void onProviderDisabled(String provider) {

				}

				@Override
				public void onProviderEnabled(String provider) {
				}

				@Override
				public void onStatusChanged(String provider, int status,
						Bundle extras) {

				}

			};
			listeners.add(ll);
			locationManager.requestLocationUpdates(providers.get(i), 60000, 25,
					ll);
		}
		
		//Calling private function to set location
		setLastKnownLocation();
	}

	public void stopUpdates() {
		LocationManager locationManager = (LocationManager) this.context
				.getSystemService(Context.LOCATION_SERVICE);
		Iterator<LocationListener> ll = listeners.iterator();
		while (ll.hasNext()) {
			locationManager.removeUpdates(ll.next());
		}
	}

	public void setLastKnownLocation() {
		
		//Checking if gps location is available
		LocationManager locationManager = (LocationManager) this.context
				.getSystemService(Context.LOCATION_SERVICE);
		Location location = locationManager.getLastKnownLocation("gps");
		
		//If gps is not available retrieve location from network
		if (location == null) {
			location = locationManager.getLastKnownLocation("network");
		}
		update(location);
	}

	public void update(Location location) {
		
		//We get location, setting point
		if (location != null) {
			Double lat = location.getLatitude() * 1E6;
			Double lng = location.getLongitude() * 1E6;
			this.point = new GeoPoint(lat.intValue(), lng.intValue());
			handler.sendEmptyMessage(LOCATION_CHANGED);
		} 
		else //Location is not available, centering in center of catalonia 
		{
			Double lat = 41.530754 * 1E6;
			Double lng = 1.679613 * 1E6;
			this.point = new GeoPoint(lat.intValue(), lng.intValue());
			handler.sendEmptyMessage(LOCATION_CHANGED);
		}
	}

	public void setRadius(int meters) {
		this.radiusInMeters = meters;
	}

	public int getRadius() {
		return this.radiusInMeters;
	}

	public GeoPoint getPoint() {
		return this.point;
	}

	@Override
	public boolean draw(Canvas canvas, MapView mapView, boolean shadow,
			long when ) {

		try {
			
			Projection astral = mapView.getProjection();
			Point screenPixels = astral.toPixels(this.point, null);
			
			this.radiusInPixels = astral
					.metersToEquatorPixels(this.radiusInMeters);
			this.centerXInPixels = screenPixels.x;
			this.centerYInPixels = screenPixels.y;

			//Setting draw parameters
			Paint paint = new Paint();
			paint.setARGB(100, 147, 186, 228);
			paint.setStrokeWidth(2);
			paint.setAntiAlias(true);
			paint.setStyle(Paint.Style.STROKE);
			
			//Drawing the outline of the circle
			canvas.drawCircle(screenPixels.x, screenPixels.y,
					this.radiusInPixels, paint);

			paint.setStyle(Paint.Style.FILL);
			paint.setAlpha(20);
			
			//Drawing the fill circle
			canvas.drawCircle(screenPixels.x, screenPixels.y,
					this.radiusInPixels, paint);
			
			//Drawing arrow from center to the outline
			drawArrow(canvas, screenPixels, this.radiusInPixels, angle);
		} catch (Exception e) {

		}
		return super.draw(canvas, mapView, shadow, when);
	}

	public void drawArrow(Canvas canvas, Point sPC, float length, double angle) {
		
		//Setting arrow parameters
		Paint paint = new Paint();
		paint.setARGB(255, 147, 186, 228);
		paint.setStrokeWidth(2);
		paint.setAntiAlias(true);
		paint.setStrokeCap(Cap.ROUND);
		paint.setStyle(Paint.Style.FILL);
		
		//Calculating x and y coordinates of the outline extreme
		float x = (float) (sPC.x + length * Math.cos(angle));
		float y = (float) (sPC.y + length * Math.sin(angle));
		
		//Drawing the line
		canvas.drawLine(sPC.x, sPC.y, x, y, paint);

		//Drawing the small circle in the center of big circle
		canvas.drawCircle(sPC.x, sPC.y, 5, paint);

		smallCircleX = x;
		smallCircleY = y;

		//Drawing small circle in the extreme of the line
		canvas.drawCircle(x, y, 8, paint);

	}

	@Override
	public boolean onTap(GeoPoint p, MapView mapView) {
		Log.d(TAG,"onTap(): entering on tap");
		return super.onTap(p, mapView);
	}

	@Override
	public boolean onTouchEvent(MotionEvent e, MapView mapView) {
		
		//Coordinates of point tapped by user
		float x = e.getX();
		float y = e.getY();

		int action = e.getAction();

		//Controlling is user tap is on small circle
		boolean onCircle = isOnCircle(x, y, this.smallCircleX,
				this.smallCircleY, this.smallCircleRadius + 20);

		
		switch (action) {
		//If user action is down
		case MotionEvent.ACTION_DOWN:
			if (onCircle) {
				this.status = 1;
			} else
				this.status = 0;
			break;
		//If user action is up
		case MotionEvent.ACTION_UP:
			if (this.status == 1) {
				this.status = 0;
				handler.sendEmptyMessage(MOTION_STOP);
			}
			break;
		//If user action is move around screen
		case MotionEvent.ACTION_MOVE:
			
			//User is moving pressing down the screen
			if (this.status == 1) {
				
				//To calculate new radius, we apply pitagoras theorem
				//dist = sqt (b2 +c2)
				double dist = Math.sqrt(Math.pow(Math.abs(this.centerXInPixels
						- x), 2)
						+ Math.pow(Math.abs(this.centerYInPixels - y), 2));
				
				//Calculating radius in meters
				this.radiusInMeters = (int) ((int) (dist * this.radiusInMeters) / this.radiusInPixels);

				// Recalculating angle
				float opp = this.centerYInPixels - y;
				float adj = this.centerXInPixels - x;
				
				//Calculating tangent with opposite and adjacent sides 
				float tan = Math.abs(opp) / Math.abs(adj);
				this.angle = (float) Math.atan(tan);
				if (opp > 0) {

					if (adj > 0) {
						this.angle += Math.PI;
					} else {
						this.angle = this.angle * -1;
					}
				} else {
					if (adj > 0) {
						this.angle = (float) Math.PI - this.angle;
					} 
				}
				//Sending motion stop message
				handler.sendEmptyMessage(MOTION_STOP);
			}
			break;
		}
		return this.status == 1;
	}
	
	
	//Function which checks if coordinates are inside the circle
	public static final boolean isOnCircle(float x, float y, float centerX,
			float centerY, double radius) {
		double square_dist = Math.pow(centerX - x, 2)
				+ Math.pow(centerY - y, 2);
		return square_dist <= Math.pow(radius, 2);
	}
	
}