package edu.uoc.rural;

import java.io.InputStream;
import java.net.URL;

import edu.uoc.rural.R;
import edu.uoc.rural.data.House;
import edu.uoc.rural.data.ListHouses;

import android.app.Activity;
import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.Gallery;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

public class Photos extends Activity {

	private ImageView imgView;
	private String TAG = "Photos";
	
	// Adding images.
	private String[] listPhotos = null;
	private Drawable[] drbl = null;
	
	private Gallery gallery;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.photos);
				
		//Recovering parameters from main tab house
	    Bundle bundle = this.getIntent().getExtras();
	    int houseId = bundle.getInt("houseId", 0);
        
		//Finding house by Id in houses list
	    ListHouses listHouses = new ListHouses(this.getApplicationContext());
	    House house = listHouses.findById(houseId);
        
	    
	    //Checking if info is populated
	    if(house.arePhotosPopulated()){
	        //Getting Urls of photos
	        listPhotos = house.getPhotos();
	        
	        //Creating array to put photo URLS
	        drbl = new Drawable[listPhotos.length];
	        
	        //Getting photo URLS from web
	        for (int i = 0; i < drbl.length; i++){
	            drbl[i] = getImageFromWeb(listPhotos[i]);
	        }
			
	        //Initializing big image ImageView
			imgView = (ImageView)findViewById(R.id.bigImage);
			//Setting first photo
			imgView.setImageDrawable(drbl[0]);
		
			//Initializing gallery ImageView
			gallery = (Gallery) findViewById(R.id.photoGallery);
			//Setting adapter
			gallery.setAdapter(new AddImgAdapter(this));
		
			gallery.setOnItemClickListener(new OnItemClickListener() {	
				public void onItemClick(AdapterView parent, View v, int position, long id) {
				// Displaying the position when the gallery item in clicked
				imgView.setImageDrawable(drbl[position]);
				}
			});
	    }else{
	    	
	    	//Showing no information available text
	    	TextView infoText=new TextView(this);
			infoText.setText("Informació no disponible");
			infoText.setTypeface(null, Typeface.BOLD);
			
			this.setContentView(infoText);
			
	    	Log.i(TAG,"onCreate(): the aren't photos for this house");
	    }
	}

private Drawable getImageFromWeb(String url){
	Drawable drb = null;
	try{
		InputStream is = (InputStream)new URL(url).getContent();
		drb = Drawable.createFromStream(is, "src name");
	}catch(Exception e){
		Log.e(TAG,"error getting image"+e.toString());
		return null;
	}
	return drb;
}

//Class Adapter to show image gallery
public class AddImgAdapter extends BaseAdapter {
	int GalItemBg;
	private Context cont;
	
	public AddImgAdapter(Context c) {
		cont = c;
		TypedArray typArray = obtainStyledAttributes(R.styleable.GalleryTheme);
		GalItemBg = typArray.getResourceId(R.styleable.GalleryTheme_android_galleryItemBackground, 0);
		typArray.recycle();
	}
	
	public int getCount() {
		return drbl.length;
	}
	
	public Object getItem(int position) {
		return position;
	}
	
	public long getItemId(int position) {
		return position;
	}
	
	public View getView(int position, View convertView, ViewGroup parent) {
		ImageView imgView = new ImageView(cont);
	
		imgView.setImageDrawable(drbl[position]);
		// Fixing width & height for image to display
		imgView.setLayoutParams(new Gallery.LayoutParams(90, 70));
		imgView.setScaleType(ImageView.ScaleType.CENTER_INSIDE);
		imgView.setBackgroundResource(GalItemBg);
		
		return imgView;
		}
	}
}