package edu.uoc.rural;

import edu.uoc.rural.R;
import edu.uoc.rural.data.House;
import edu.uoc.rural.data.ListHouses;
import edu.uoc.rural.util.Services;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.app.TabActivity;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.widget.TabHost;
import android.widget.TabHost.TabSpec;

public class TabHouse extends TabActivity {
	
	private static String TAG = "TabHouse";

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.tabhouse);
        
		//Recovering parameters from main tabhouse
	    Bundle bundle = this.getIntent().getExtras();
	    int houseId = bundle.getInt("houseId", 0);
        
		//Finding house by id in houses list
	    ListHouses listHouses = new ListHouses(this.getApplicationContext());
	    House house = listHouses.findById(houseId);
        
        try{
        	//Calling async task to load house data is its empty
        	if(!house.isInfoPopulated()){
	        		new LoadHouse().execute(house);
        	}
        	else //Creating tab host is house is not empty
        	{
        		createTabHost(houseId);
        	}
        }catch(Exception e){
        	Log.e(TAG,"onCreate(): error creating anb initializing services");
        	showAlert();
        }      
    }
    
    //This function creates tab host with four tabs
    private void createTabHost(int houseId){
    	try{
		    //Creating TabHost
		    TabHost tabHost = (TabHost)findViewById(android.R.id.tabhost);
	
		    //TabSpec to create tabs
		    TabSpec ts_info = tabHost.newTabSpec("tid_info");
		    TabSpec ts_photos = tabHost.newTabSpec("tid_photos");
		    TabSpec ts_amenities = tabHost.newTabSpec("tid_amenities");
		    TabSpec ts_prices = tabHost.newTabSpec("tid_prices");
		    
		    
		    //Creatting tab Intents
		    Intent intentInfo = new Intent(this,Info.class);
		    Intent intentPhotos = new Intent(this,Photos.class);
		    Intent intentAmenities = new Intent(this,Amenity.class);
		    Intent intentPrices = new Intent(this,Prices.class);
		    
		    //Passing house name to tabs using intents
		    Bundle bund = new Bundle();
			bund.putInt("houseId",houseId);
			intentInfo.putExtras(bund);
			intentPhotos.putExtras(bund);
			intentAmenities.putExtras(bund);
			intentPrices.putExtras(bund);
		    
		    //Setting tab names
		    ts_info.setIndicator("Info").setContent(intentInfo);
		    ts_photos.setIndicator("Fotos").setContent(intentPhotos);
		    ts_amenities.setIndicator("Serveis").setContent(intentAmenities);
		    ts_prices.setIndicator("Preus").setContent(intentPrices);
		
		    //Add tabSpec to the TabHost to display
		    tabHost.addTab(ts_info);
		    tabHost.addTab(ts_photos);
		    tabHost.addTab(ts_amenities);
		    tabHost.addTab(ts_prices);
    	}catch(Exception e){
			 Log.e(TAG,"Error"+e.toString());
		 }
    }
   
    //This class loads house data in async mode
    private class LoadHouse extends AsyncTask<House,Void,Integer>{
    	
		private ProgressDialog progDialog = new ProgressDialog(TabHouse.this);
		House house = null;
		 
		//This function is called before main task and creates and show progress dialog
		protected void onPreExecute() {
			//Creating progress dialog
			progDialog.setMessage("Carregant. Espereu si us plau ... ");
			progDialog.show();
        }
		
		//Main task which populates house info (photos, amenities, and more)
		protected Integer doInBackground(House... h) {
			 try{
				 Services s = new Services();
				 house = (House)h[0];
				 //Checking if list is empty or had been populated before
		        if(!house.isInfoPopulated()){
		            house.populateInfo(s);	
		        }
		     }catch(Exception e){
		        Log.e(TAG,"run(): error running async house populate"+e.toString());
		        cancel(true);
		        progDialog.dismiss();
		        showAlert();
		        return new Integer(-1);
		     }
		     return new Integer(house.getId());
	     }

		//Function called after main task, creates tab host
	     protected void onPostExecute(Integer houseId) {
	    	 progDialog.dismiss();
	    	 if(house.isInfoPopulated()){
	    		 createTabHost(houseId.intValue());
	    	 }else{
	    		 showAlert();
	    	 }
	    	 
	     }

	}
    
    private void showAlert(){
    	AlertDialog alert = new AlertDialog.Builder(this).create();
		alert.setTitle("Avís: error obtenint dades");
		alert.setMessage("S'ha produit un error obtenint les dades ");
		alert.setButton("Tanca", new DialogInterface.OnClickListener() {
		   public void onClick(DialogInterface dialog, int which) {
	    		 finish();
		   }
		});
		alert.setIcon(R.drawable.icon);
		alert.show();
    }
    
}