package edu.uoc.rural.data;

import java.util.Enumeration;
import java.util.Hashtable;

import java.util.Vector;

import android.content.Context;
import android.util.Log;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.Overlay;

import edu.uoc.rural.Maps.MapHouseItem;
import edu.uoc.rural.util.Services;

public class House extends Overlay {
	
	private static String TAG = "House";
	
	private int id;
	private Context context;
	private double distance;
	private String name = "";
	private String description = "";
	private String address = "";
	private String phone = "";
	private String email = "";
	private String[] photos;
	private Hashtable<String,String> amenities;
	private Hashtable<String,String> pricesWeekend;
	private Hashtable<String,String> pricesWeek;
	private Hashtable<String,String> seasons;
	private MapHouseItem houseOverlay;
	private boolean isBasicInfoPopulated;
	private boolean isInfoPopulated;
	private boolean arePhotosPopulated;
	private boolean areAmenitiesPopulated;
	private boolean arePricesPopulated;
	
	public House(int id){
		this.id = id;
	}
	
	public House (int id, GeoPoint geopoint, Context context){
		this.id = id;
		this.context = context;
		this.houseOverlay = new MapHouseItem(geopoint,"",""); 
		this.isInfoPopulated=false;
		this.areAmenitiesPopulated=false;
		this.arePhotosPopulated=false;
		this.arePricesPopulated=false;
		this.isBasicInfoPopulated=false;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public Context getContext() {
		return context;
	}

	public void setContext(Context context) {
		this.context = context;
	}

	public double getDistance() {
		return distance;
	}

	public void setDistance(double distance) {
		this.distance = distance;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}
	
	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public String getPhone() {
		return phone;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String[] getPhotos() {
		return photos;
	}

	public void setPhotos(String[] photos) {
		this.photos = photos;
	}
	
	public Hashtable<String, String> getAmenities() {
		return amenities;
	}

	public void setAmenities(Hashtable<String, String> amenities) {
		this.amenities = amenities;
	}
	
	public Hashtable<String, String> getPricesWeekend() {
		return pricesWeekend;
	}

	public void setPricesWeekend(Hashtable<String, String> priceWeekend) {
		this.pricesWeekend = priceWeekend;
	}

	public Hashtable<String, String> getPricesWeek() {
		return pricesWeek;
	}

	public void setPricesWeek(Hashtable<String, String> priceWeek) {
		this.pricesWeek = priceWeek;
	}
	
	public Hashtable<String,String> getSeasons() {
		return seasons;
	}

	public void setSeasons(Hashtable<String,String> seasons) {
		this.seasons = seasons;
	}

	public MapHouseItem getHouseOverlay() {
		return houseOverlay;
	}

	public void setHousesOverlay(MapHouseItem houseOverlay) {
		this.houseOverlay = houseOverlay;
	}
	
	public boolean isInfoPopulated() {
		return isInfoPopulated;
	}

	public void setInfoPopulated(boolean isBasicInfoPopulated) {
		this.isInfoPopulated = isBasicInfoPopulated;
	}

	public boolean arePhotosPopulated() {
		return arePhotosPopulated;
	}

	public void setArePhotosPopulated(boolean arePhotosPopulated) {
		this.arePhotosPopulated = arePhotosPopulated;
	}

	public boolean areAmenitiesPopulated() {
		return areAmenitiesPopulated;
	}

	public void setAreAmenitiesPopulated(boolean areAmenitiesPopulated) {
		this.areAmenitiesPopulated = areAmenitiesPopulated;
	}

	public boolean arePricesPopulated() {
		return arePricesPopulated;
	}

	public void setArePricesPopulated(boolean arePricesPopulated) {
		this.arePricesPopulated = arePricesPopulated;
	}

	public boolean isBasicInfoPopulated() {
		return isBasicInfoPopulated;
	}

	public void setBasicInfoPopulated(boolean isBasicInfoPopulated) {
		this.isBasicInfoPopulated = isBasicInfoPopulated;
	}

	public void populateInfo(Services s){
		Hashtable<Integer, Hashtable<String, Object>> houseFields = null;
		Vector<Integer> ids = new Vector<Integer>();
		
		//Getting house id
		ids.add(this.getId());
		
		//Creating vectors with query fields
		Vector<String> fields = new Vector<String>();
		fields.add("address_id");
		fields.add("description");
		fields.add("total_places");	// Integer
		fields.add("total_additional_places");	// Integer
		fields.add("total_n_rooms");	// Integer
		fields.add("total_n_additional_rooms");	// Integer
		fields.add("total_n_bathrooms");	// Integer
		fields.add("image_ids");
		fields.add("amenity_line_ids");
		fields.add("season_ids");
		
		
		//Reading fields from server
		try{
			houseFields = s.read("houserenting.house", ids, fields);
		}catch(Exception e){
			Log.e(TAG,"populate(): error getting field values"+e.toString());
		}
		Log.d(TAG, "houseFields: "+houseFields);
		
		if (houseFields!=null) {
			try {
				
				//Getting and populating address, phone and e-mail
				if (houseFields.get(this.getId()).get("address_id")!=null && 
						houseFields.get(this.getId()).get("address_id").getClass()==Object[].class) 
				{
					Log.d(TAG, "address_id: "+(Object[]) houseFields.get(this.getId()).get("address_id"));
					Hashtable<Integer, Hashtable<String, Object>> addressFields = null;
					
					
					
					Integer addressId = (Integer) ((Object[]) houseFields.get(this.getId()).get("address_id"))[0];
					ids = new Vector<Integer>();
					ids.add(addressId);
					fields = new Vector<String>();
					fields.add("phone");
					fields.add("email");
					
					//Reading phone and email fields from address
					try {
						addressFields = s.read("res.partner.address", ids, fields);
					} catch(Exception e) {
						Log.e(TAG,"address.populate(): error getting field values"+e.toString());
					}
					this.address = (String) ((Object[])houseFields.get(this.getId()).get("address_id"))[1];
					this.phone = (String) addressFields.get(addressId).get("phone");
					this.email = (String) addressFields.get(addressId).get("email");
					
				}
				
				//Getting and populating house description
				if (houseFields.get(this.getId()).get("description")!=null && 
						houseFields.get(this.getId()).get("description").getClass()==String.class)
				{
					this.description = (String) houseFields.get(this.getId()).get("description");
				}
				
				//Getting and populating photos
				if (houseFields.get(this.getId()).get("image_ids")!=null && 
						houseFields.get(this.getId()).get("image_ids").getClass()==Object[].class) 
				{
					this.populateImages(s, (Object[]) houseFields.get(this.getId()).get("image_ids"));
					this.arePhotosPopulated=true;
				}
				
				//Getting and populating amenities
				if (houseFields.get(this.getId()).get("amenity_line_ids")!=null && 
						houseFields.get(this.getId()).get("amenity_line_ids").getClass()==Object[].class) 
				{
					this.populateAmenities(s, (Object[]) houseFields.get(this.getId()).get("amenity_line_ids"));
					this.areAmenitiesPopulated=true;
				}
				
				//Getting and populating seasons
				if (houseFields.get(this.getId()).get("season_ids")!=null && 
						houseFields.get(this.getId()).get("season_ids").getClass()==Object[].class) 
				{
					this.populateSeasons(s, (Object[]) houseFields.get(this.getId()).get("season_ids"));
					this.arePricesPopulated=true;
				}
				this.isInfoPopulated=true;
			}catch(Exception e){
				Log.e(TAG,"populate():"+e.toString());
			}
		}else{
			Log.e(TAG,"populate(): error getting addres from openerp");
		}
	}
	
	//Function which reads and populates images of a house
	private void populateImages(Services s, Object[] image_ids) {
		if (s==null || image_ids==null) return;
		
		Log.d(TAG, "A fotos");
		Hashtable<Integer,Hashtable<String,Object>> imageFields = null;
		Vector<Integer> ids = new Vector<Integer>();
		for (int itImg=0; itImg<image_ids.length; itImg++) {
			ids.add( (Integer) image_ids[itImg] );
		}
		
		Vector<String> fields = new Vector<String>();
		fields.add("filename");
		try {
			imageFields = s.read("product.images", ids, fields);
		}catch(Exception e) {
			Log.e(TAG,"address.populate(): error getting field values"+e.toString());
		}
		
		this.photos = new String[ids.size()];
		for (int itImg=0; itImg<ids.size(); itImg++) 
		{
			photos[itImg] = "http://rc.proves.argus.net/docroot/ruralcatalunya/includes/houses/"+ (String) imageFields.get(ids.get(itImg)).get("filename");
		}
	}
	
	//This function populates house amenities
	private void populateAmenities(Services s, Object[] amenity_line_ids) {
		if (s==null || amenity_line_ids==null) return;
		

		Hashtable<Integer,Hashtable<String,Object>> amenityLineFields = null;
		
		//We have all amenity ids and put them in a vector
		Vector<Integer> ids = new Vector<Integer>();
		for (int itImg=0; itImg<amenity_line_ids.length; itImg++) {
			ids.add( (Integer) amenity_line_ids[itImg] );
		}
		
		//Creating vector with fields that will be read
		Vector<String> fields = new Vector<String>();
		fields.add("amenity_id");
		fields.add("included");
		fields.add("quantity");
		fields.add("fixed_price");
		fields.add("perday_price");
		fields.add("perperson_price");
		
		//Reading fields from house
		try {
			amenityLineFields = s.read("houserenting.house.amenity_line", ids, fields);
		}catch(Exception e) {
			Log.e(TAG,"amenities.populate(): error getting field values"+e.toString());
		}
		
		this.amenities = new Hashtable<String,String>();
		if (amenityLineFields!=null) {
			Enumeration<Integer> amKeys = amenityLineFields.keys();
			while(amKeys.hasMoreElements()) {
				Integer amKey = amKeys.nextElement();
				String amenityName = (String) ((Object[]) amenityLineFields.get(amKey).get("amenity_id"))[1];
				String amenityDescription = "";
				if ( ((Boolean) amenityLineFields.get(amKey).get("included")).booleanValue() ) {
					//Amenity has a determined quantity
					if ( ((Integer) amenityLineFields.get(amKey).get("quantity")).intValue()>0 )
						amenityDescription = ((Integer) amenityLineFields.get(amKey).get("quantity")).intValue()+"";
					//Amenity its only yes or not value
					else
						amenityDescription = "Sí";
				} else {
					//Amenity has a fixed price independent of day number
					if ( ((Double) amenityLineFields.get(amKey).get("fixed_price")).doubleValue()>0 ){
						amenityDescription = ((Double) amenityLineFields.get(amKey).get("fixed_price")).doubleValue()+"€ per reserva";
					}
					//Amenity has a perday price
					if ( ((Double) amenityLineFields.get(amKey).get("perday_price")).doubleValue()>0 ) {
						if (amenityDescription.length()>0) amenityDescription += ", ";
						amenityDescription = ((Double) amenityLineFields.get(amKey).get("perday_price")).doubleValue()+"€ per dia";
					}
					//Amenity has a price for each person
					if ( ((Double) amenityLineFields.get(amKey).get("perperson_price")).doubleValue()>0 ) {
						if (amenityDescription.length()>0) amenityDescription += ", ";
						amenityDescription = ((Double) amenityLineFields.get(amKey).get("perperson_price")).doubleValue()+"€ per persona/dia";
					}
				}
				this.amenities.put(amenityName, amenityDescription);
			}
		}
	}
	
	//This function populates house with prices for week and weekend days for each season	
	private void populateSeasons(Services s, Object[] seasons_ids) {
		if (s==null || seasons_ids==null) return;
		
		//Processing parameter season_ids to put ids in a Vector
		Vector<Integer> ids = new Vector<Integer>();
		for (int itSeason=0; itSeason<seasons_ids.length; itSeason++) {
			ids.add( (Integer) seasons_ids[itSeason] );
		}
		
		//Creating vectors with fields to read from openerp
		Vector<String> fields = new Vector<String>();
		fields.add("pricelist_id");
		fields.add("weekend_list_price");
		fields.add("week_list_price");
		
		//Calling read function to get data from openerp
		Hashtable<Integer,Hashtable<String,Object>> seasonFields = null;
		try {
			seasonFields = s.read("houserenting.house.season", ids, fields);
		}catch(Exception e) {
			Log.e(TAG,"populateSeasons(): error reading field values from openerp"+e.toString());
		}
		
		
		seasons = new Hashtable<String,String>();
		this.pricesWeek = new Hashtable<String,String>();
		this.pricesWeekend = new Hashtable<String,String>();
		
		//Processing data to get week and weekend prices separately in two hash tables
		if (seasonFields!=null) {
			Enumeration<Integer> seaKeys = seasonFields.keys();
			while(seaKeys.hasMoreElements()) {
				Integer amKey = seaKeys.nextElement();
				String seasonName = (String) ((Object[]) seasonFields.get(amKey).get("pricelist_id"))[1];
				seasonName = seasonName.replace(" (EUR)", "");
				seasons.put(seasonName,"");
				
				//Populating vectors with prices for week and weekend days
				this.pricesWeekend.put(seasonName, ((Double) seasonFields.get(amKey).get("weekend_list_price")).doubleValue()+"€");
				this.pricesWeek.put(seasonName, ((Double) seasonFields.get(amKey).get("week_list_price")).doubleValue()+"€");
			}
		}
	}
}
