package edu.uoc.rural.data;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import android.os.Handler;

import com.google.android.maps.GeoPoint;
import edu.uoc.rural.Maps.MapHouseItem;
import edu.uoc.rural.util.MathHelper;
import edu.uoc.rural.util.Services;

import android.content.Context;
import android.util.Log;

public class ListHouses {
	
	public static List<House> houses = null;
	private static String TAG = "ListHouses";
	private Context context;
	private static String order;

	public ListHouses(Context context){
		this.context = context;
		order = "not_ordered";
		return;
	}

	public void createList (Services s, Handler handler) {
		if(houses==null){
			houses = new ArrayList<House>();
			order = "not_ordered";
		}
	}
	
	public List<House> getList(){
		return houses;
	}
	
	public void addHouse (House house){
		houses.add(house);
	}
	
	public boolean isEmpty(){
		return houses == null || houses.size() == 0;
	}
	
	public Vector<Integer> getIds(){
		
		Vector<Integer> ids = null;
		
		if(houses!=null && houses.size()>0){
			ids = new Vector<Integer>();
			
			for(int i=0;i<houses.size();i++){
				ids.add((Integer)houses.get(i).getId());
			}
		}
		return ids;
	}
	
	//Returns house with position in the list
	public House getHouse(int position){
		House house = null;
		if(houses.get(position) instanceof House){
			house = houses.get(position);
		}
		return house;
	}
	
	//Returns house with id
	public House findById(int id){
		Iterator<House> it = houses.iterator();
		House house;
		House aux;
		while(it.hasNext()){
			aux = it.next();
			if(aux.getId() == id){
				house = (House)aux;
				return house;
			}
		}
		return null;
	}
	
	//Returns house with passed name
	public House findByName(String name){
		Iterator<House> it = houses.iterator();
		House house;
		House aux;
		while(it.hasNext()){
			aux = it.next();
			if(aux.getName().equals(name)){
				house = (House)aux;
				return house;
			}
		}
		return null;
	}
	
	public void populateBasicInfo(Services s){
		Vector<Integer> houseIds = null;
		
		//Getting house id from openerp
		try {
			houseIds = getHouseIdsFromDB(s);
		}catch(Exception e){
			Log.e(TAG,"populateList: error getting houses"+e.toString());
			return;
		}

		Hashtable<Integer, Hashtable<String, Object>> houseFields = null;
		
		//Creating vectors with query fields
		Vector<String> fields = new Vector<String>();
		fields.add("name");
		fields.add("geocode_longitude");
		fields.add("geocode_latitude");
		
		//Reading fields from server
		try{
			houseFields = s.read("houserenting.house", houseIds, fields);
		}catch(Exception e){
			Log.e(TAG,"setMapHouses(): error getting field values"+e.toString());
		}
		
		if (houseFields!=null) {
			Iterator<Integer> houseIdsIt = houseIds.iterator();
			while (houseIdsIt.hasNext()) {
				Integer id = houseIdsIt.next();
				
				//Creating house object
				House house = new House(id);
				
				//Setting name, latitude and longitude attributes
				house.setName((String) houseFields.get(id).get("name"));
				Double latitude = (Double) houseFields.get(id).get("geocode_latitude") *1E6;
				Double longitude = (Double) houseFields.get(id).get("geocode_longitude") *1E6;

				//Creating house overlay. It's a MapHouseItem.
				MapHouseItem houseItem = new MapHouseItem(new GeoPoint(latitude.intValue(),longitude.intValue()),"","");
				
				//Setting id of the house to item
				houseItem.setId(house.getId());
				house.setHousesOverlay(houseItem);
				house.setBasicInfoPopulated(true);
				
				//Adding house to list
				houses.add(house);
				
			}
		}
	}
	
	//Method which returns vector with house names
	public String[] getNames(){
		
		String[] names = null;
		if(houses!= null && houses.size()>0){
			
			names = new String[houses.size()];
			Iterator<House> it = houses.iterator();
			
			for(int i=0;i<houses.size();i++){
				names[i] = it.next().getName();
			}
		}
		return names;
	}
	
	
	
	public List<MapHouseItem> getNearHousesOverlay(int Radius){
		
		List<MapHouseItem> nearHouses = null;
		
		int marginError = 15000;
		
		for(int i=0;i<houses.size();i++){
			//Getting house
			House h = houses.get(i);

			
			//Calculating if distance is higher than radius
			if(h.getDistance() + marginError <Radius)
			{
				Log.i(TAG,"House ditance is"+h.getDistance()+"and radius is "+Radius+"and gap is "+(h.getDistance()+marginError));
				if(nearHouses==null){
					nearHouses = new ArrayList<MapHouseItem>();
				}
				nearHouses.add(h.getHouseOverlay());
			}
		}
		return nearHouses;
	}
	
	//Method to update houses distance from user location
	public void updateDistances(GeoPoint point){
		
		for(int i=0;i<houses.size();i++){
			
			//Getting house
			House h = houses.get(i);
			
			//Setting new distance between house and user position
			h.setDistance(MathHelper.gp2m(point,h.getHouseOverlay().getGeopoint()));
		}
	}
	
	//This class implements a comparator to compare Houses by Distance
	public class HouseComparerByDistance implements Comparator<House> {
		
		 public int compare(House house1, House house2){
			 if(house1==null) return 1;
			 if(house2==null) return -1;
			 double v1 = house1.getDistance();
			 double v2 = house2.getDistance();
			 if(v1>v2){
				 return 1;
			 }else{
				 return -1;
			 }
		 }
	}
	 
	//This class implements a comparator to compare Houses by Name
	public class HouseComparerByName implements Comparator<House> {
			
		 public int compare(House house1, House house2){
			 if(house1==null) return 1;
			 if(house2==null) return -1;
			 String name1 = house1.getName();
			 String name2 = house2.getName();
			 return name1.compareToIgnoreCase(name2);
		 }
	}
	
	//Method to order house list by name using comparator
	public void orderListByName(){
		//Sort list by name
		if(houses == null){
			return;
		}else if (order.equals("not_ordered") || !order.equals("byName")){
			Comparator<House> compName = new HouseComparerByName();
			Collections.sort(houses,compName);
			order="byName";
		}
	}
	
	//Method to order house list by distance using comparator
	public void orderListByDistance(){
		//Sort list by distance
		if(houses == null){
			return;
		}else if(order.equals("not_ordered") || !order.equals("byDistance")){
	        Comparator<House> compDistance = new HouseComparerByDistance();
	        Collections.sort(houses,compDistance);
	        order="byDistance";
		}
	}
	
	//Method which loads houses ids from openerp database
	private Vector<Integer> getHouseIdsFromDB(Services s){
		
		Vector<Integer> houseIds = null;
		
		try{
			//Calling search with empty Vector returns all houses ids from database
			houseIds = s.search("houserenting.house", new Vector<Vector<String>>());
			
			//Checking if result is empty or not
			if(houseIds == null || houseIds.size() <= 0){
				Log.d(TAG,"getHouseNames(): Houses not found");
			}
		}catch(Exception e){
			Log.e(TAG,"getHouseIdsFromDB(): Error getting house Ids"+e.toString());
		}	
		return houseIds;
	}
}
