package edu.uoc.rural.util;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Vector;

import android.util.Log;

import org.xmlrpc.android.*;

public class Services {

	public static int OP_EQ = 1;		// =
	public static int OP_NOTEQ = 2;		// !=
	public static int OP_LT = 3;		// <
	public static int OP_GT = 4;		// >
	public static int OP_LE = 5;		// <=
	public static int OP_GE = 6;		// >=
	public static int OP_IN = 7;		// in
	public static int OP_NOTIN = 8;		// not in
	public static int OP_LIKE = 9;		// like
	public static int OP_ILIKE = 10;		// ilike
	public static int OP_CHILDOF = 11;	// child_of
	
	private String m_protocol = "http";
	private String m_host="example.host.com";
	private int m_port = 8169;
	private String m_database;
	private String m_user;
	private int m_userId = 0;
	private String m_passwd;
	private String m_model;
	private String TAG = "Services";
	
	
	public Services() throws Exception{
		m_userId = 0;
		this.setUrl("examnple.host.com");
		this.setDataBase("database_name");
		this.setUser("username");
		this.setPasswd("password");
		this.setModel("houserenting.house");
		boolean correcte=false;
		try{
			correcte = this.login();
			Log.d(TAG,"El valor de login és"+correcte);
		}catch(Exception e){
			Log.e(TAG,"El valor de login és"+correcte);
			e.printStackTrace();
		}
	}
	
	/**
	 * Sets the supplied protocol for future connections.
	 * If 'protocol' is null or empty, it puts the standard protocol ('http').
	 * @param protocol The protocol to use in future connections.
	 * @throws Exception If 'protocol' is not 'http' or 'https'.
	 */
	public void setProtocol(String protocol) throws Exception {
		if (protocol==null || protocol.length() == 0) m_protocol = "http";
		else if (!protocol.equals("http") && !protocol.equals("https")) throw new Exception("INVALID_PROTOCOL");
		else m_protocol=protocol;
	}
	public String getProtocol() {
		return m_protocol;
	}
	
	/**
	 * Sets the supplied host for future connections.
	 * @param host The host whre is the OpenERP server. It can't be null or empty
	 * @throws Exception If 'host' parameter is null or empty.
	 */
	public void setHost(String host) throws Exception {
		if (host==null || host.length() == 0) throw new Exception("EMPTY_HOST");
		m_host=host;
	}
	public String getHost() {
		return m_host;
	}
	
	/**
	 * Sets the supplied port for future connections.
	 * If 'protocol' parameter is 0 or -1, it puts the standard port ('8169').
	 * @param port The port to use in future connections.
	 */
	public void setPort(int port) {
		if (port<=0) m_port = 8169;
		else m_port=port;
	}
	public int getPort() {
		return m_port;
	}
	
	/**
	 * Sets the 'protocol', 'host' and 'port' attributes.
	 * If 'protocol' or 'port' is not specified, it puts the defaults values ('http' and '8169').
	 * @param url The XML-RPC(S) port url of OpenERP server. Something link http://localhost:8169
	 * @throws Exception
	 */
	public void setUrl(String url) throws Exception {
		if (url==null || url.length() == 0) throw new Exception("EMPTY_URL");
		Log.d(TAG,"setUrl: url="+url);
		
		if (url.indexOf("://")==0) throw new Exception("EMPTY_PROTOCOL");
		if (url.indexOf("://")>0) {
			setProtocol(url.substring(0, url.indexOf("://")));
			url = url.substring(url.indexOf("://")+3);
		}
		else setProtocol(null);
		
		if (url.contains("/")) {
			Log.w(TAG,"setUrl: url contains rpc action="+url);
			url = url.substring(0, url.indexOf("/"));
		}
		
		int port = 0;
		if (url.indexOf(":")>0) {
			try {
				port = Integer.parseInt(url.substring(url.indexOf(":")+1));
				url = url.substring(0, url.indexOf(":"));
			} catch (NumberFormatException e) {
				Log.d(TAG,"setUrl(): NumberFormatException exception parsing the port : '"+url.substring(url.indexOf(":")+1)+"'");
				throw new Exception("INVALID_PROTOCOL : '"+url.substring(url.indexOf(":")+1)+"'");
			}
		}
		setPort(port);
		
		setHost(url);
		Log.d(TAG,"setUrl: m_protocol="+m_protocol+", m_host="+m_host+", m_port="+m_port);
	}
	
	/**
	 * Sets the supplied database for future connections.
	 * @param bd
	 * @throws Exception
	 */
	public void setDataBase(String bd) throws Exception {
		if (bd==null || bd.length() == 0) throw new Exception("EMPTY_DATABASE");
		m_database = bd;
	}
	public String getDataBase() {
		return m_database;
	}
	
	/**
	 * Sets the supplied user for future connections (login action).
	 * @param user
	 * @throws Exception 
	 */
	public void setUser(String user) throws Exception {
		if (user==null || user.length() == 0) throw new Exception("EMPTY_USER");
		m_user = user;
	}
	public String getUser() {
		return m_user;
	}
	
	/**
	 * Sets the password for furture connections (login action).
	 * @param pass
	 * @throws Exception 
	 */
	public void setPasswd(String pass) throws Exception {
		if (pass==null || pass.length() == 0) throw new Exception("EMPTY_PASSWD");
		m_passwd = pass;
	}
	public String getPasswd() {
		return m_passwd;
	}
	
	/**
	 * Returns the user ID of last login action.
	 * If user not logged or not logged correctly, it is 0. 
	 * @return The logged user ID, or 0 if not logged 
	 */
	public int getUserId() {
		return m_userId;
	}
	
	/**
	 * Sets the model for future actions.
	 * @param model
	 * @throws Exception
	 */
	public void setModel(String model) throws Exception {
		if (model==null || model.length() == 0) throw new Exception("EMPTY_MODEL");
		m_model = model;
	}
	public String getModel() {
		return m_model;
	}
	
	/**
	 * Connect to the OpenERP defined with {@link #setUrl(String)} or {@link #setHost(String)} 
	 * and try to login to database with user and password supplied defined with {@link #setDataBase(String)},  
	 * {@link #setUser(String)} and {@link #setPasswd(String)}.
	 * If you get identified correctly puts the user ID to the 'userId' attribute (you can get it with #getUserId
	 * @return If logged in correctly.
	 * @throws Exception 
	 */
	public boolean login() throws Exception {
		checkConnectionAndLoginData();
		
		//Creating XML-RPC client
		XMLRPCClient client = getClient("/xmlrpc/common");
		
		Log.d(TAG,"login(): Executing xmlrpc login with params: {"+m_database+", "+m_user+", "+m_passwd+"}");
		
		Object result;
		try {
			//Calling login function with authentication params
			result = (Object) client.callEx("login", new Object[] {m_database,m_user, m_passwd});
			
			if (result instanceof Integer) {
				m_userId = ((Integer) result).intValue();
				return true;
			} 
			m_userId=0;
			return false;
		} catch (XMLRPCException e) {
			Log.e(TAG,"login():"+e.toString());
			throw new Exception("CONNECTION_ERROR", e);
		} catch (Exception e) {
			Log.e(TAG,"login()"+e.toString());
			throw new Exception("CONNECTION_ERROR", e);
		}
	}
	
	/**
	 * 
	 * @param model
	 * @param filter
	 * @return
	 * @throws Exception
	 * @throws Exception Model 'model' doesn't exists.
	 * @throws Exception Parameter 'model' or 'filter' are null
	 */
	public Vector<Integer> search(String model, Vector<Vector<String>> filter) throws Exception {
		checkModelExists(model);
		if (filter==null) throw new Exception("FILTER_NOT_SUPPLIED");
		
		XMLRPCClient client = getClient("/xmlrpc/object" );
		
		Object[] params = new Object[] {m_database,m_userId, m_passwd,model,"search",filter};
		
		try {
			Object[] result = (Object[]) client.callEx("execute", params);
			
			Vector<Integer> ret = new Vector<Integer>();
			if (result!=null && result.length>0 && result[0] instanceof Integer) {
				for (int i=0; result!=null && i<result.length; i++)
					ret.add((Integer) result[i]);
			}
			else if (result==null || result.length!=0)
				Log.d(TAG,"search(): The result isn't an array of Integers : "+result);
			return ret;
		} catch (XMLRPCException e) {
			Log.d(TAG,"search(): XMLRPCException exception executing the 'search' action over '"+model+"' with filter="+filter);
			throw new Exception("CONNECTION_ERROR", e);
		} catch (Exception e) {
			Log.d(TAG,"search(): Exception exception executing the 'search' action over '"+model+"' with filter="+filter);
			throw new Exception("CONNECTION_ERROR", e);
		}
	}
		
	/**
	 * 
	 * @param model
	 * @param ids
	 * @param fields
	 * @return
	 * @throws Exception
	 * @throws Exception Model 'model' doesn't exists.
	 * @throws Exception Parameter 'model', 'ids' or 'fields' are null
	 */
	@SuppressWarnings("unchecked")
	public Hashtable<Integer, Hashtable<String, Object>> read(String model, Vector<Integer> ids, Vector<String> fields) throws Exception {
		
		//Checking model
		checkModelExists(model);
		
		//Checking parameters
		if (ids==null) throw new Exception("IDS_NOT_SUPPLIED");
		if (ids.size()==0) return null;
		if (fields==null) throw new Exception("FIELDS_NOT_SUPPLIED");
		
		//Creating XML-RPC client
		XMLRPCClient client = getClient("/xmlrpc/object");
		
		//Creating params
		Object[] params = new Object[] {m_database,m_userId, m_passwd,model,"read",ids,fields};
		
		try {
			//Executing read function
			Object[] result = (Object[]) client.callEx("execute", params);
			
			Hashtable<Integer, Hashtable<String, Object>> ret = new Hashtable<Integer, Hashtable<String, Object>>();
			if (result!=null && result.length>0 && result[0] instanceof HashMap) {
				for (int i=0; result!=null && i<result.length; i++) {
					Hashtable<String, Object> aux = new Hashtable<String, Object>();
					aux.putAll( (HashMap<String, Object>) result[i] );
					ret.put((Integer) aux.get("id"), aux);
				}
			}
			else if (result==null || result.length!=0)
				Log.d(TAG,"read(): The result isn't an array of HashMap : "+result);
			
			return ret;
		} catch (XMLRPCException e) {
			Log.d(TAG,"read(): XMLRPCException exception executing the 'read' action with ids="+ids+" and fields="+fields);
			throw new Exception("CONNECTION_ERROR", e);
		} catch (Exception e) {
			Log.d(TAG,"read(): Exception exception executing the 'read' action with ids="+ids+" and fields="+fields);
			throw new Exception("CONNECTION_ERROR", e);
		}
	}
	
	/**
	 * 
	 * @param rpcString
	 * @return
	 * @throws Exception HOST_NOT_DEFINED or INVALID_CONNECTON_DATA
	 */
	private XMLRPCClient getClient(String rpcString) throws Exception {
		if (m_host==null || m_host.length() == 0) throw new Exception("HOST_NOT_DEFINED");	
		
		XMLRPCClient client = null;
	
		try {
			client = new XMLRPCClient("http://example.host.com:8169"+rpcString,"username","password");
		} catch (Exception e) {
			throw new Exception("INVALID_CONNECTON_DATA", e);
		}
		
		return client;
	}
	
	private void checkConnectionAndLoginData() throws Exception {
		if (m_host==null || m_host.length() == 0) {
			Log.d(TAG,"checkConnectionAndLoginData(): m_host ('"+m_host+"') is null or empty.");
			throw new Exception("HOST_NOT_DEFINED");
		}
		if (m_database==null || m_database.length() == 0 || m_user==null || m_user.length() == 0 || m_passwd==null || m_passwd.length() == 0) {
			Log.d(TAG,"checkConnectionAndLoginData(): m_database ('"+m_database+"'), m_user ('"+m_user+"') or m_passwd ('"+m_passwd+"') are null or empty.");
			throw new Exception("NO_LOGIN_DATA");
		}
	}
	
	/**
	 * 
	 * @param model The model name to check if exists
	 * @throws Exception Connection error. Normally, invalid login credentials.
	 * @throws Exception Model 'model' doesn't exists
	 * @throws Exception Parameter 'model' is null
	 */
	private void checkModelExists(String model) throws Exception {
		if (model==null) throw new Exception("MODEL_NOT_SUPPLIED");
		
		XMLRPCClient client = getClient("/xmlrpc/object" );
		
		if(client==null) throw new Exception("null client");
		
		Object[] result = null;
		try {
			result = (Object[]) client.callEx("execute", new Object[] {m_database, m_userId, m_passwd, "ir.model", "search", new Object[]{new Object[]{"model","=",model}}});
		} catch (XMLRPCException e) {
			throw new Exception("CONNECTION_ERROR", e);
		} catch (Exception e) {
			throw new Exception("CONNECTION_ERROR", e);
		}
		
		if (result==null || result.length<=0) throw new Exception("model : "+model);
	}
		
	public boolean valid_operator(int operator){
		return (operator>1 && operator<12);
	}
	
	
	public String operatorString(int operator){
		if(!valid_operator(operator)) return "";
		
		if(operator==OP_EQ) return "=";
		else if(operator==OP_NOTEQ) return "!=";
		else if(operator==OP_LT) return "<";
		else if(operator==OP_GT) return ">";
		else if(operator==OP_LE) return "<=";
		else if(operator==OP_GE) return ">=";
		else if(operator==OP_IN) return "in";
		else if(operator==OP_NOTIN) return "not in";
		else if(operator==OP_LIKE) return "like";
		else if(operator==OP_ILIKE) return "ilike";
		else if(operator==OP_CHILDOF) return "child_of";		
		return "";
	}
}
